@@ -29,17 +29,27 @@ extern char restore_rt_sigaction;
 int arch_compat_rt_sigaction(void *stack32, int sig, rt_sigaction_t_compat *act)
 {
 	int ret;
+	struct syscall_args32 arg = {};
+	unsigned long act_stack = (unsigned long)stack32;
+
+	/* To make sure the 32-bit stack was allocated in caller */
+	if (act_stack >= (uint32_t)-1) {
+		pr_err("compat rt_sigaction without 32-bit stack\n");
+		return -1;
+	}
 
 	/*
 	 * To be sure, that sigaction pointer lies under 4G,
 	 * coping it on the bottom of the stack.
 	 */
 	memcpy(stack32, act, sizeof(rt_sigaction_t_compat));
+	arg.nr		= __NR32_rt_sigaction;
+	arg.arg0	= sig;
+	arg.arg1	= (uint32_t)act_stack;			/* act */
+	arg.arg2	= 0;					/* oldact */
+	arg.arg3	= (uint32_t)sizeof(act->rt_sa_mask);	/* sigsetsize */
 
-	asm volatile ("\t movl %%ebx,%%ebx\n" : :"b"(sig));	/* signum */
-	asm volatile ("\t movl %%ecx,%%ecx\n" : :"c"(stack32));	/* act */
-	asm volatile ("\t movl %%edx,%%edx\n" : :"d"(sizeof(act->rt_sa_mask)));
-	call32_from_64(stack32 + PAGE_SIZE, &restore_rt_sigaction);
+	do_full_int80(&arg);
 	asm volatile ("\t movl %%eax,%0\n" : "=r"(ret));
 	return ret;
 }
