@@ -735,6 +735,7 @@ static struct collect_image_info remap_cinfo = {
 static int dump_ghost_file(int _fd, u32 id, const struct stat *st, dev_t phys_dev)
 {
 	struct cr_img *img;
+	int exit_code = -1;
 	GhostFileEntry gfe = GHOST_FILE_ENTRY__INIT;
 	Timeval atim = TIMEVAL__INIT, mtim = TIMEVAL__INIT;
 
@@ -771,7 +772,7 @@ static int dump_ghost_file(int _fd, u32 id, const struct stat *st, dev_t phys_de
 	}
 
 	if (pb_write_one(img, &gfe, PB_GHOST_FILE))
-		return -1;
+		goto err_out;
 
 	if (S_ISREG(st->st_mode)) {
 		int fd, ret;
@@ -785,7 +786,7 @@ static int dump_ghost_file(int _fd, u32 id, const struct stat *st, dev_t phys_de
 		fd = open(lpath, O_RDONLY);
 		if (fd < 0) {
 			pr_perror("Can't open ghost original file");
-			return -1;
+			goto err_out;
 		}
 
 		if (gfe.chunks)
@@ -794,11 +795,13 @@ static int dump_ghost_file(int _fd, u32 id, const struct stat *st, dev_t phys_de
 			ret = copy_file(fd, img_raw_fd(img), st->st_size);
 		close(fd);
 		if (ret)
-			return -1;
+			goto err_out;
 	}
 
+	exit_code = 0;
+err_out:
 	close_image(img);
-	return 0;
+	return exit_code;
 }
 
 struct file_remap *lookup_ghost_remap(u32 dev, u32 ino)
@@ -1553,6 +1556,9 @@ static int rfi_remap(struct reg_file_info *rfi, int *level)
 	}
 
 	mi = lookup_mnt_id(rfi->rfe->mnt_id);
+	if (mi == NULL)
+		return -1;
+
 	if (rfi->rfe->mnt_id == rfi->remap->rmnt_id) {
 		/* Both links on the same mount point */
 		tmi = mi;
@@ -1562,6 +1568,8 @@ static int rfi_remap(struct reg_file_info *rfi, int *level)
 	}
 
 	rmi = lookup_mnt_id(rfi->remap->rmnt_id);
+	if (rmi == NULL)
+		return -1;
 
 	/*
 	 * Find the common bind-mount. We know that one mount point was
@@ -1704,12 +1712,10 @@ ext:
 		}
 
 		if (rfi->rfe->has_mode && (st.st_mode != rfi->rfe->mode)) {
-			if (st.st_mode != rfi->rfe->mode) {
-				pr_err("File %s has bad mode 0%o (expect 0%o)\n",
-				       rfi->path, (int)st.st_mode,
-				       rfi->rfe->mode);
-				return -1;
-			}
+			pr_err("File %s has bad mode 0%o (expect 0%o)\n",
+			       rfi->path, (int)st.st_mode,
+			       rfi->rfe->mode);
+			return -1;
 		}
 
 		/*
