@@ -138,6 +138,12 @@ struct cr_options {
 	pid_t			tree_id;
 	int			log_level;
 	char			*imgs_dir;
+	char			*tls_cacert;
+	char			*tls_cacrl;
+	char			*tls_cert;
+	char			*tls_key;
+	int			tls;
+	int			tls_no_cn_verify;
 };
 
 extern struct cr_options opts;
