@@ -3,6 +3,7 @@
 
 #include "images/inventory.pb-c.h"
 #include "images/creds.pb-c.h"
+#include "images/fdinfo.pb-c.h"
 
 #define AA_SECURITYFS_PATH "/sys/kernel/security/apparmor"
 
@@ -33,5 +34,22 @@ int validate_lsm(char *profile);
  */
 int render_lsm_profile(char *profile, char **val);
 
-extern int parse_lsm_arg(char *arg);
+extern int lsm_check_opts(void);
+
+#ifdef CONFIG_HAS_SELINUX
+int dump_xattr_security_selinux(int fd, FdinfoEntry *e);
+int run_setsockcreatecon(FdinfoEntry *e);
+int reset_setsockcreatecon();
+#else
+static inline int dump_xattr_security_selinux(int fd, FdinfoEntry *e) {
+	return 0;
+}
+static inline int run_setsockcreatecon(FdinfoEntry *e) {
+	return 0;
+}
+static inline int reset_setsockcreatecon() {
+	return 0;
+}
+#endif
+
 #endif /* __CR_LSM_H__ */
