@@ -14,6 +14,18 @@ struct ns_id;
 
 #define MNT_UNREACHABLE INT_MIN
 
+/*
+ * We have remounted these mount writable temporary, and we
+ * should return it back to readonly at the end of file restore.
+ */
+#define REMOUNTED_RW 1
+/*
+ * We have remounted these mount writable in service mount namespace,
+ * thus we shouldn't return it back to readonly, as service mntns
+ * will be destroyed anyway.
+ */
+#define REMOUNTED_RW_SERVICE 2
+
 struct mount_info {
 	int			mnt_id;
 	int			parent_mnt_id;
@@ -65,9 +77,14 @@ struct mount_info {
 	struct list_head	mnt_slave_list;	/* list of slave mounts */
 	struct list_head	mnt_slave;	/* slave list entry */
 	struct mount_info	*mnt_master;	/* slave is on master->mnt_slave_list */
+	struct list_head	mnt_propagate;	/* circular list of mounts which propagate from each other */
+	struct list_head	mnt_notprop;	/* temporary list used in can_mount_now */
 
 	struct list_head	postpone;
 
+	int			is_overmounted;
+	int			remounted_rw;
+
 	void			*private;	/* associated filesystem data */
 };
 
@@ -124,4 +141,8 @@ extern struct mount_info *parse_mountinfo(pid_t pid, struct ns_id *nsid, bool fo
 
 extern int check_mnt_id(void);
 
+extern int remount_readonly_mounts(void);
+extern int try_remount_writable(struct mount_info *mi, bool ns);
+extern bool mnt_is_overmounted(struct mount_info *mi);
+
 #endif /* __CR_MOUNT_H__ */
